"""
Functions for plotting points.
"""

from typing import Mapping, Optional, Sequence, Union
from itertools import repeat

import staticmaps
import pygeodesy
from PIL.Image import Image


from landfall.color import process_colors, process_id_colors


tp = staticmaps.tile_provider_OSM


def plot_points(
    lats,
    lons,
    colors: Optional[Union[Sequence, str]] = None,
    ids: Optional[Sequence] = None,
    id_colors: Optional[Union[Mapping, str]] = None,
    tile_provider=tp,
    point_size=10,
    window_size=(500, 400),
    zoom=0,
    color=staticmaps.color.BLUE,
    set_zoom=None,
    center=None
) -> Image:
    context = staticmaps.Context()
    context.set_tile_provider(tile_provider)
    count = len(lats)
    if colors is not None:
        colors = process_colors(colors, count)
    else:
        colors = list(repeat(color, count))

    if ids is not None and id_colors is not None:
        colors = process_id_colors(ids, id_colors)

    for lat, lon, clr in zip(lats, lons, colors):
        point = staticmaps.create_latlng(lat, lon)
        marker = staticmaps.Marker(point, color=clr, size=point_size)
        context.add_object(marker)

    _center, _zoom = context.determine_center_zoom(*window_size)
    context.set_zoom(_zoom + zoom)

    if center is not None:
        if type(center) is tuple:
            point = staticmaps.create_latlng(*center)
        if type(center) is str:
            lat, lon = pygeodesy.geohash.decode(*center)
            point = staticmaps.create_latlng(float(lat), float(lon))
        context.set_center(point)
    if set_zoom is not None:
        context.set_zoom(set_zoom)
    return context.render_pillow(*window_size)
    

def plot_points_data(
    data: Mapping[str, Sequence],
    lat_name: str,
    lon_name: str,
    tile_provider=tp,
    point_size=10,
    window_size=(500, 400),
    zoom=0,
    color=staticmaps.color.BLUE,
    set_zoom=None,
    center=None
) -> Image:
    lats = data[lat_name]
    lons = data[lon_name]
    return plot_points(
        lats, lons,
        tile_provider=tile_provider,
        point_size=point_size,
        window_size=window_size,
        zoom=zoom,
        color=color,
        set_zoom=set_zoom,
        center=center)


import colorsys
from typing import Set, Tuple 
 
    
def hsvt_to_rgb(h, s, v) -> Tuple[int, int, int]:
    (r, g, b) = colorsys.hsv_to_rgb(h, s, v) 
    return int(r*255), int(g*255), int(b*255)
 
    
def get_wheel_colors(n) -> Set[Tuple[int, int, int]]:
    hue_partition = 1 / (n + 1)
    colors = set()
    for value in range(0, n):
        new_color = hsvt_to_rgb(hue_partition * value, .99, 1)
        if new_color in colors:
            raise ValueError('n is too high')
        if max(new_color) > 255 or min(new_color) < 0:
            raise ValueError('color is not real')
        colors.add(new_color)
    return colors


from typing import List

import distinctipy


def get_distict_colors(
    qty: int,
    pastel_factor = 0
) -> List[tuple]:
    colors = distinctipy.get_colors(qty, pastel_factor=pastel_factor)
    return [(int(r) * 255, int(g) * 255, int(b) * 255) for r, g, b in colors]


"""
Functions for using colors.
"""

import random
from typing import Sequence, List, Mapping, Union

from staticmaps import Color, parse_color

from .distinctipy import get_distict_colors
from .colorsys import get_wheel_colors


def random_color():
    r = random.randrange(0, 256)
    g = random.randrange(0, 256)
    b = random.randrange(0, 256)
    return Color(r, g, b)


def process_colors(
    colors: Sequence,
    count: int
) -> List[Color]:
    if type(colors) is str:
        if colors == 'random':
            colors = [random_color() for _ in range(count)]
        elif colors == 'distinct':
            colors = convert_colors(get_distict_colors(count))
        elif colors == 'wheel':
            colors = convert_colors(get_wheel_colors(count))
        else:
            raise ValueError('str must be "random", "distinct", or "wheel"')
    else:
        colors = convert_colors(colors)
    return colors


def convert_colors(colors) -> List[Color]:
    return [convert_color(color) for color in colors]


def convert_color(color) -> Color:
    if isinstance(color, str):
        return parse_color(color)
    if isinstance(color, Color):
        return color
    if len(color) in (3, 4):
        return Color(*color)
    else:
        raise ValueError('process_color only takes str, Color or (r, g, b)')
    

def process_id_colors(ids: Sequence, id_colors: Union[Mapping, str]) -> List[Color]:
    """Return a list of unique colors for each id."""
    if type(id_colors) is str:
        id_colors = map_id_colors(ids, id_colors)
    return [convert_color(id_colors[id]) for id in ids]


def map_id_colors(ids: Sequence, color_code: str) -> dict:
    """Map colors to each id."""
    count = len(ids)
    unique_ids = list(set(ids))
    return {id: color for id, color in zip(unique_ids, process_colors(color_code, count))}
    


